/* Copyright (C) 2014-2016 Dan Chapman <dpniel@ubuntu.com>

   This file is part of the dekko

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.1
import Dekko.Theme 0.2
import "../../Components"

Popover {
    id: info
    z: 100
    property var model
    signal remove()

    Column {
        anchors {
            left: parent.left
            top: parent.top
            right: parent.right
        }

        ListItem {
            height: layout.height
            ListItemLayout {
                id: layout
                title.text: model.name ? model.name : model.email
                title.font.weight: Style.common.fontWeight
                title.elide: Text.ElideRight
                title.wrapMode: Text.NoWrap
                subtitle.text: {
                    if (model.name) {
                        if (model.name !== model.email)
                            return model.email
                        else
                            return ""
                    } else if (model.isInContacts) {
                        return model.contactInfo.organization
                    } else {
                        return ""
                    }
                }
                summary.text: model.name && model.isInContacts ? model.contactInfo.organization : ""
                subtitle.font.weight: Style.common.fontWeight
                summary.font.weight: Style.common.fontWeight
                subtitle.elide: Text.ElideRight
                subtitle.wrapMode: Text.NoWrap

                Avatar {
                    id: avatar
                    name: model.name
                    initials: model.initials
                    email: model.email
                    validContact: true
                    fontSize: "large"
                    fontWeight: Style.common.fontWeight
                    SlotsLayout.position: SlotsLayout.Leading
                }
            }
        }

        ListItem {
            height: deleteLayout.implicitHeight
            ListItemLayout {
                id: deleteLayout
                title.text: qsTr("Remove")
                title.font.weight: Style.common.fontWeight
                title.elide: Text.ElideRight
                title.wrapMode: Text.NoWrap
            }
            onClicked: info.remove()
        }
        ListItem {
            visible: !model.isInContacts
            height: addContactLayout.implicitHeight
            ListItemLayout {
                id: addContactLayout
                title.text: qsTr("Add to addressbook")
                title.font.weight: Style.common.fontWeight
                title.elide: Text.ElideRight
                title.wrapMode: Text.NoWrap
            }
            onClicked: model.addToContacts()
        }
    }
}

