/* Copyright (C) 2014-2016 Dan Chapman <dpniel@ubuntu.com>

   This file is part of the dekko

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.1
import "../Components"
import Dekko.Theme 0.2
import Dekko.Composition 0.2

Popover {
    id: context
    z: 100

    property Action zoomAction
    property Action viewSourceAction
    property Action fullscreenAction
    property int currentZoom: 100

    Column {
        id: containerLayout
        anchors {
            left: parent.left
            top: parent.top
            right: parent.right
        }
        ListItem {
            width: parent.width
            height: forwardLayout.height
            ListItemLayout {
                id: forwardLayout
                title.text: qsTr("Forward")
                title.fontSize: "x-small"
                title.elide: Text.ElideRight
                title.wrapMode: Text.NoWrap
                title.font.weight: Font.Normal
                CachedImage {
                    id: flagIcon
                    height: units.gu(2.5)
                    width: height
                    name: "mail-forward"
                    color: Style.header.icons
                    SlotsLayout.position: SlotsLayout.Leading
                }
            }
            onClicked: {
                mainStage.replyToMessage(ReplyMode.REPLY_FORWARD)
                PopupUtils.close(context)
            }
        }
        ListItem {
            width: parent.width
            height: fullscreenLayout.height
            ListItemLayout {
                id: fullscreenLayout
                title.text: qsTr("Full screen")
                title.fontSize: "x-small"
                title.elide: Text.ElideRight
                title.wrapMode: Text.NoWrap
                title.font.weight: Font.Normal
                CachedImage {
                    height: units.gu(2.5)
                    width: height
                    name: "view-fullscreen"
                    color: Style.header.icons
                    SlotsLayout.position: SlotsLayout.Leading
                }
            }
            onClicked: {
                fullscreenAction.trigger()
                PopupUtils.close(context)
            }
        }
        ListItem {
            width: parent.width
            height: viewSourceLayout.height
            ListItemLayout {
                id: viewSourceLayout
                title.text: qsTr("View source")
                title.fontSize: "x-small"
                title.elide: Text.ElideRight
                title.wrapMode: Text.NoWrap
                title.font.weight: Font.Normal
                CachedImage {
                    height: units.gu(2.5)
                    width: height
                    name: "note"
                    color: Style.header.icons
                    SlotsLayout.position: SlotsLayout.Leading
                }
            }
            onClicked: {
                viewSourceAction.trigger()
                PopupUtils.close(context)
            }
        }

        Item {
            width: parent.width
            height: zoomLayout.height
            ListItemLayout {
                id: zoomLayout
                title.text: qsTr("Zoom: %1%").arg(currentZoom)
                title.fontSize: "x-small"
                title.elide: Text.ElideRight
                title.wrapMode: Text.NoWrap
                title.font.weight: Font.Normal
                Button {
                    height: units.gu(3.5)
                    width: height
                    strokeColor: "#ffffff"
                    SlotsLayout.position: SlotsLayout.Leading
                    onClicked: {
                        if (currentZoom > 50) {
                            currentZoom -= 10
                            zoomAction.trigger(currentZoom)
                        }
                    }
                    Icon {
                        height: units.gu(2.5)
                        width: height
                        name: "zoom-out"
                        anchors.centerIn: parent
                        color: Style.header.icons
                    }
                }
                Button {
                    height: units.gu(3.5)
                    width: height
                    strokeColor: "#ffffff"
                    onClicked: {
                        if (currentZoom < 200) {
                            currentZoom += 10
                            zoomAction.trigger(currentZoom)
                        }
                    }
                    Icon {
                        height: units.gu(2.5)
                        width: height
                        name: "zoom-in"
                        anchors.centerIn: parent
                        color: Style.header.icons
                    }
                }
            }
        }
    }
}

